
/*
 *                            COPYRIGHT
 *
 *  XAnalyser, frequence spectrum analyser for X Windows
 *  Copyright (C) 1998 Arvin Schnell
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Contact addresses:
 *  aschnell@physik.uni-bremen.de or arvinschnell@yahoo.com
 *  Arvin Schnell, Am Heidberg 8, 28865 Lilienthal, Germany
 *
 */


#include <iostream.h>
#include <iomanip.h>
#include <stdio.h>
#include <ctype.h>

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/Scale.h>
#include <Xm/RowColumn.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/Frame.h>

#include "Trans.hh"
#include "main.hh"
#include "control.hh"
#include "display.hh"
#include "help.hh"
#include "calc.hh"
#include "misc.hh"



Widget card_w, disk_w;
Widget mono_w, left_w, right_w, stereo_w;
Widget alpha_w, decay_w;
Widget search_w, marker_w, peak_w;
Widget run_w;


#define XWORKPROC

#ifdef XWORKPROC
XtWorkProcId workproc_id;
#else
XtIntervalId interval_id;
int wait;
#endif

int run = 0;


void
quit_cb( Widget, XtPointer, XtPointer )
{
  if( run )
    calc_close();
  
  XtDestroyApplicationContext( app_context );
  
  exit( 0 );
}



void
set_sensitivity()
{
  if( run && source != SOUNDCARD )
    XtSetSensitive( card_w, False );
  else
    XtSetSensitive( card_w, True );
 
  if( run && source != DISKFILE )
    XtSetSensitive( disk_w, False );
  else
    XtSetSensitive( disk_w, True );
  
  if( searchon ) {
    XtSetSensitive( marker_w, True );
    XtSetSensitive( peak_w, True );
  } else {
    XtSetSensitive( marker_w, False );
    XtSetSensitive( peak_w, False );
  }
  
  if( run )
    XmToggleButtonSetState( run_w, True, False );
  else
    XmToggleButtonSetState( run_w, False, False );
}



void
update_marker_label()
{
  const int size = 20;
  char buffer[size];
  
  float f1, f2;
  K.X.BtoA( marker_x[0], f1 );
  K.X.BtoA( marker_x[1], f2 );
  
#ifdef _GNU_SOURCE
  snprintf( buffer, size, "%5d Hz - %5d Hz", (int) f1, (int) f2 );
#else
  sprintf( buffer, "%5d Hz - %5d Hz", (int) f1, (int) f2 );
#endif
  
  XmString xmstr = XmStringCreateLocalized( buffer );
  Arg args[] = {
    { XmNlabelString, (XtArgVal) xmstr }
  };
  XtSetValues( marker_w, args, XtNumber( args ) );
  XmStringFree( xmstr );
}



void
update_peak_label()
{
  const int size = 20;
  char buffer[size];
  
  float f;
  K.X.BtoA( peak_x, f );
  
#ifdef _GNU_SOURCE
  snprintf( buffer, size, "%5d Hz %5.1f dB", (int) f, peak_db );
#else
  sprintf( buffer, "%5d Hz %5.1f dB", (int) f, peak_db );
#endif
  
  XmString xmstr = XmStringCreateLocalized( buffer );
  Arg args[] = {
    { XmNlabelString, (XtArgVal) xmstr }
  };
  XtSetValues( peak_w, args, XtNumber( args ) );
  XmStringFree( xmstr );
}


#ifndef _BSD_SOURCE
int
strcasecmp( const char *s1, const char *s2 )
{
  int l = min( strlen( s1 ), strlen( s2 ) );
  for( int i = 0; i < l; i++ ) {
    int ret = toupper( s1[i] ) != toupper( s2[i] );
    if( ret != 0 ) return ret;
  }
  return 0;
}
#endif



void
control_set_default()
{
  if( strcasecmp( xanalyser.source, "SOUNDCARD" ) == 0 ) {
    source = SOUNDCARD;
    XmToggleButtonSetState( card_w, True, False );
  };
  if( strcasecmp( xanalyser.source, "DISKFILE" ) == 0 ) {
    source = DISKFILE;
    XmToggleButtonSetState( disk_w, True, False );
  }
  
  
  if( strcasecmp( xanalyser.channels, "MONO" ) == 0 ) {
    channels = MONO;
    XmToggleButtonSetState( mono_w, True, False );
  }
  if( strcasecmp( xanalyser.channels, "LEFT" ) == 0 ) {
    channels = LEFT;
    XmToggleButtonSetState( left_w, True, False );
  }
  if( strcasecmp( xanalyser.channels, "RIGHT" ) == 0 ) {
    channels = RIGHT;
    XmToggleButtonSetState( right_w, True, False );
  }
  if( strcasecmp( xanalyser.channels, "STEREO" ) == 0 ) {
    channels = STEREO;
    XmToggleButtonSetState( stereo_w, True, False );
  }
  
  
  alpha = xanalyser.alpha;
  myXmScaleSetValue( alpha_w, alpha );
  
  
  decay = xanalyser.decay;
  myXmScaleSetValue( decay_w, decay );
}



#ifdef XWORKPROC
Boolean
workproc( XtPointer )
{
  if( calc_shot() ) {
    
    if( frame_count % (int)(frames_per_second) == 0 && searchon ) {
      calc_search_peak();
      update_peak_label();
    }
  
    return FALSE;
    
  } else {
    
    calc_close();
    run = 0;
    
    set_sensitivity();
    
    return TRUE;
    
  }
}
#else
void
timeout( XtPointer, XtIntervalId * )
{
  if( calc_shot() ) {
    
    if( frame_count % (int)(frames_per_second) == 0 && searchon ) {
      calc_search_peak();
      update_peak_label();
    }
    
    // call myself
    
    interval_id = XtAppAddTimeOut( app_context, wait,
				   timeout, (XtPointer) 0 );
    
  } else {
    
    calc_close();
    run = 0;
    
    set_sensitivity();
  }
}
#endif



void
control_start()
{
  if( !run )
    if( calc_open() ) {
      
#ifdef XWORKPROC
      workproc_id = XtAppAddWorkProc( app_context, workproc, NULL );
#else
      wait = (int)( 1000 / frames_per_second * 0.10 );   // good ?
      interval_id = XtAppAddTimeOut( app_context, wait,
				     timeout, (XtPointer) 0 );
#endif
      
      run = 1;
    }
  
  set_sensitivity();
}



void
control_stop()
{
  if( run ) {
#ifdef XWORKPROC
    XtRemoveWorkProc( workproc_id );
#else
    XtRemoveTimeOut( interval_id );
#endif
    
    calc_close();
    run = 0;
  }
  
  set_sensitivity();
}



void
control_callback( Widget widget, XtPointer client_data, XtPointer )
{
  int i = (int) client_data;
  switch( i ) {
    
  case SOUNDCARD:
  case DISKFILE:    
    source = i;
    calc_parms();
    break;
    
  case MONO:
  case LEFT:
  case RIGHT:
  case STEREO:
    channels = i;
    calc_parms();
    break;
    
  case DECAY:
    decay = myXmScaleGetValue( widget );
    calc_parms();
    break;
    
  case ALPHA:
    alpha = myXmScaleGetValue( widget );
    calc_parms();
    break;
    
  case SEARCH:
    searchon = !searchon;
    calc_search_peak();
    update_marker_label();
    update_peak_label();
    set_sensitivity();
    calc_parms();
    break;

  case HELP:
    help_manage();
    break;
    
  case RUN:
    if( !run )
      control_start();
    else
      control_stop();
    break;
    
  }
}



void
control_create()
{
  Arg args[10];
  int n;
  
  
  // control
  
  Widget control_w;
  
  n = 0;
  control_w = XmCreateForm( toplevel, "Control", args, n );
  XtManageChild( control_w );
  
  
  // frame1 
  
  Widget frame1_w, form1_w, box1_w, label11_w;
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  frame1_w = XmCreateFrame( control_w, "Frame1", args, n );
  XtManageChild( frame1_w );
  
  
  // form1
  
  n = 0;
  form1_w = XmCreateForm( frame1_w, "Form1", args, n );
  XtManageChild( form1_w );
  
  
  // label: source
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNrightAttachment, XmATTACH_FORM ); n++;
  label11_w = XmCreateLabel( form1_w, "Source", args, n );
  XtManageChild( label11_w );
  
  
  // radiobox1
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNtopWidget, label11_w ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  box1_w = XmCreateRadioBox( form1_w, "RadioBox1", args, n );
  XtManageChild( box1_w );
  
  
  // radio button: card
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  card_w = XmCreateToggleButton( box1_w, "Soundcard", args, n );
  XtManageChild( card_w );
  XtAddCallback( card_w, XmNvalueChangedCallback,
		 control_callback, (XtPointer) SOUNDCARD );
  
  
  // radio button: disk
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNtopWidget, card_w ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  disk_w = XmCreateToggleButton( box1_w, "Diskfile", args, n );
  XtManageChild( disk_w );
  XtAddCallback( disk_w, XmNvalueChangedCallback,
		 control_callback, (XtPointer) DISKFILE );
  
  
  // frame2
  
  Widget frame2_w, form2_w, label21_w, box2_w;
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNleftWidget, frame1_w ); n++;
  XtSetArg( args[n], XmNbottomAttachment, XmATTACH_FORM ); n++;
  frame2_w = XmCreateFrame( control_w, "Frame2", args, n );
  XtManageChild( frame2_w );
  
  
  // form2
  
  n = 0;
  form2_w = XmCreateForm( frame2_w, "Form2", args, n );
  XtManageChild( form2_w );
  
  
  // label: channels
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNrightAttachment, XmATTACH_FORM ); n++;
  label21_w = XmCreateLabel( form2_w, "Channels", args, n );
  XtManageChild( label21_w );
  
  
  // radiobox2
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNtopWidget, label21_w ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  box2_w = XmCreateRadioBox( form2_w, "RadioBox2", args, n );
  XtManageChild( box2_w );
  
  
  // radio button: mono
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  mono_w = XmCreateToggleButton( box2_w, "Mono", args, n );
  XtManageChild( mono_w );
  XtAddCallback( mono_w, XmNvalueChangedCallback,
		 control_callback, (XtPointer) MONO );
  
 
  // radio button: left
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNtopWidget, mono_w ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  left_w = XmCreateToggleButton( box2_w, "Left", args, n );
  XtManageChild( left_w );
  XtAddCallback( left_w, XmNvalueChangedCallback,
		 control_callback, (XtPointer) LEFT );
  
  
  // radio button: right
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNtopWidget, left_w ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  right_w = XmCreateToggleButton( box2_w, "Right", args, n );
  XtManageChild( right_w );
  XtAddCallback( right_w, XmNvalueChangedCallback,
		 control_callback, (XtPointer) RIGHT );
  
   
  // radio button: stereo
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNtopWidget, right_w ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  stereo_w = XmCreateToggleButton( box2_w, "Stereo", args, n );
  XtManageChild( stereo_w );
  XtAddCallback( stereo_w, XmNvalueChangedCallback,
		 control_callback, (XtPointer) STEREO );
  
  
  // frame3
  
  Widget frame3_w, form3_w, label31_w;
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNleftWidget, frame2_w ); n++;
  XtSetArg( args[n], XmNbottomAttachment, XmATTACH_FORM ); n++;
  frame3_w = XmCreateFrame( control_w, "Frame3", args, n );
  XtManageChild( frame3_w );
  
  
  // form3
  
  n = 0;
  form3_w = XmCreateForm( frame3_w, "Form3", args, n );
  XtManageChild( form3_w );
  
  
  // label: Parameter
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNrightAttachment, XmATTACH_FORM ); n++;
  label31_w = XmCreateLabel( form3_w, "Parameters", args, n );
  XtManageChild( label31_w );
  
  
  // scale: alpha
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNtopWidget, label31_w ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNrightAttachment, XmATTACH_FORM ); n++;
  alpha_w = XmCreateScale( form3_w, "Alpha", args, n );
  XtManageChild( alpha_w );
  XtAddCallback( alpha_w, XmNdragCallback, control_callback, (XtPointer) ALPHA );
  XtAddCallback( alpha_w, XmNvalueChangedCallback,
		 control_callback, (XtPointer) ALPHA );
  
  
  // scale: decay
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNtopWidget, alpha_w ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNrightAttachment, XmATTACH_FORM ); n++;
  decay_w = XmCreateScale( form3_w, "Decay", args, n );
  XtManageChild( decay_w );
  XtAddCallback( decay_w, XmNdragCallback, control_callback, (XtPointer) DECAY );
  XtAddCallback( decay_w, XmNvalueChangedCallback,
		 control_callback, (XtPointer) DECAY );
  
  
  // frame4
  
  Widget frame4_w, form4_w, label41_w, search_w;
    
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNleftWidget, frame3_w ); n++;
  XtSetArg( args[n], XmNrightAttachment, XmATTACH_FORM ); n++;
  frame4_w = XmCreateFrame( control_w, "Frame4", args, n );
  XtManageChild( frame4_w );
  
  
  // form4
  
  n = 0;
  form4_w = XmCreateForm( frame4_w, "Form4", args, n );
  XtManageChild( form4_w );
  
  
  // label: Peak Search
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNrightAttachment, XmATTACH_FORM ); n++;
  label41_w = XmCreateLabel( form4_w, "Peak Search", args, n );
  XtManageChild( label41_w );
  
  
  // toggle: search
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNtopWidget, label41_w ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  search_w = XmCreateToggleButton( form4_w, "Search", args, n );
  XtManageChild( search_w );
  XtAddCallback( search_w, XmNvalueChangedCallback,
		 control_callback, (XtPointer) SEARCH );
  
  
  // label: marker
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNtopWidget, search_w ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNrightAttachment, XmATTACH_FORM ); n++;
  
  marker_w = XmCreateLabelGadget( form4_w, "Marker", args, n );
  XtManageChild( marker_w );
  
  
  // label: peak
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNtopWidget, marker_w ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNrightAttachment, XmATTACH_FORM ); n++;
  
  peak_w = XmCreateLabelGadget( form4_w, "Peak", args, n );
  XtManageChild( peak_w );
    
  
  // frame5
  
  Widget frame5_w, form5_w, label51_w, quit_w;
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNtopWidget, frame4_w ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNleftWidget, frame3_w ); n++;
  XtSetArg( args[n], XmNbottomAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNrightAttachment, XmATTACH_FORM ); n++;
  frame5_w = XmCreateFrame( control_w, "Frame5", args, n );
  XtManageChild( frame5_w );
  
  
  // form5
  
  n = 0;
  form5_w = XmCreateForm( frame5_w, "Form5", args, n );
  XtManageChild( form5_w );
  
  
  // label: Action
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNrightAttachment, XmATTACH_FORM ); n++;
  label51_w = XmCreateLabel( form5_w, "Action", args, n );
  XtManageChild( label51_w );
  
  
  // toggle: analyser
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNtopWidget, label51_w ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  run_w = XmCreateToggleButton( form5_w, "Run", args, n );
  XtManageChild( run_w );
  XtAddCallback( run_w, XmNvalueChangedCallback,
		 control_callback, (XtPointer) RUN );
  
  
  // button: quit
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNtopWidget, label51_w ); n++;
  XtSetArg( args[n], XmNrightAttachment, XmATTACH_FORM ); n++;
  quit_w = XmCreatePushButton( form5_w, "Quit", args, n  );
  XtManageChild( quit_w );
  XtAddCallback( quit_w, XmNactivateCallback, quit_cb, 0 );
  
    
  // frame6
  
  Widget frame6_w, form6_w, label61_w, help_w;
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNtopWidget, frame1_w ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNbottomAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNrightAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNrightWidget, frame2_w ); n++;
  frame6_w = XmCreateFrame( control_w, "Frame6", args, n );
  XtManageChild( frame6_w );
  
  
  // form6
  
  n = 0;
  form6_w = XmCreateForm( frame6_w, "Form6", args, n );
  XtManageChild( form6_w );
  
  
  // label: help
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNrightAttachment, XmATTACH_FORM ); n++;
  label61_w = XmCreateLabel( form6_w, "Help", args, n );
  XtManageChild( label61_w );
  
  
  // button: help
  
  n = 0;
  XtSetArg( args[n], XmNtopAttachment, XmATTACH_WIDGET ); n++;
  XtSetArg( args[n], XmNtopWidget, label61_w ); n++;
  XtSetArg( args[n], XmNleftAttachment, XmATTACH_FORM ); n++;
  XtSetArg( args[n], XmNrightAttachment, XmATTACH_FORM ); n++;
  help_w = XmCreatePushButton( form6_w, "Help", args, n  );
  XtManageChild( help_w );
  XtAddCallback( help_w, XmNactivateCallback,
		 control_callback, (XtPointer) HELP );
  
}
